"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var Observable_1 = require("rxjs/Observable");
require("rxjs/add/observable/fromPromise");
var base_component_1 = require("./base.component");
var global_constants_1 = require("../../shared/global.constants");
var bit_toolbar_component_1 = require("../../shared/component/bit-toolbar.component");
var bitmessage_1 = require("../../shared/data/bitmessage");
var error_1 = require("../../shared/data/error");
var custom_validators_service_1 = require("../../shared/service/custom-validators.service");
var general_utils_service_1 = require("../../shared/service/general-utils.service");
/**
 * Clase base para implementar formularios de elementos. Provee el 95% de la funcionalidad necesaria.
 */
var BaseFormComponent = (function (_super) {
    __extends(BaseFormComponent, _super);
    /**
     * Constructor de la Clase.
     * @param dataService: Servicio que implementa la interface BaseCRUDServiceInterface.
     * @param router: Objeto de tipo Router para poder navegar (p. ej., al listado).
     * @param activateRoute: Objeto de tipo ActivatedRoute que permite acceder a los parámetros de la URL con los que
     *        hemos accedido a este componente o para navegar.
     * @param notificationService: Objeto de tipo NotificationService para poder mostrar diálogos al usuario.
     */
    function BaseFormComponent(dataService, router, activatedRoute, messageService, toolbarService, componentId) {
        var _this = _super.call(this, messageService, toolbarService, componentId) || this;
        _this.breadcrumb = [];
        _this.onSave = new core_1.EventEmitter();
        _this.myTabs = {};
        _this.loadedTab = {};
        _this.dataService = dataService;
        _this.router = router;
        _this.activatedRoute = activatedRoute;
        _this.toolbar = {
            //mínimos toolbar de un formulario
            "save": { visible: true, enable: true },
            "delete": { visible: false, enable: true }
        };
        // mensajes para el footer
        // mensajes para el footer
        _this.messages = new Map();
        _this.messages.set(global_constants_1.PageStatus.Init, "Premi el botó Desa per actualitzar les dades");
        _this.messages.set(global_constants_1.PageStatus.Searching, "Enviant la sol·licitud...");
        _this.messages.set(global_constants_1.PageStatus.FinishSearch, "Dades actualitzades satisfactòriament");
        _this.messages.set(global_constants_1.PageStatus.ReadOnly, "Formulari no editable");
        return _this;
    }
    BaseFormComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        console.log("ngAfterContentInit base-form.component");
        this.reset && this.reset.subscribe(function (x) {
            _this.object = null;
            _this.myForm && _this.myForm.markAsPristine(); // el formulario vuelve a estar sincronizado con el servidor
            _this.myForm && _this.myForm.markAsUntouched(); // idem
            _this.init();
        });
        this.initTabs();
    };
    /**
     * Inicializa el formulario.
     * 1. Identifica si el formulario se carga para insertar un nuevo elemento o consultar/actualizar uno existente.
     * 2. En este segundo caso, lee el elemento con el identificador indicado por parámetro desde el servidor.
     * 3. Una vez cargado el elemento, invoca el método postInit con el resultado devuelto por el servidor.
     * @param id: Identificador del elemento. En el caso de una inserción, puede ser cero o undefined.
     */
    /**
       * Inicializa el formulario.
       * 1. Identifica si el formulario se carga para insertar un nuevo elemento o consultar/actualizar uno existente.
       * 2. En este segundo caso, lee el elemento con el identificador indicado por parámetro desde el servidor.
       * 3. Una vez cargado el elemento, invoca el método postInit con el resultado devuelto por el servidor.
       * @param id: Identificador del elemento. En el caso de una inserción, puede ser cero o undefined.
       */
    BaseFormComponent.prototype.init = /**
       * Inicializa el formulario.
       * 1. Identifica si el formulario se carga para insertar un nuevo elemento o consultar/actualizar uno existente.
       * 2. En este segundo caso, lee el elemento con el identificador indicado por parámetro desde el servidor.
       * 3. Una vez cargado el elemento, invoca el método postInit con el resultado devuelto por el servidor.
       * @param id: Identificador del elemento. En el caso de una inserción, puede ser cero o undefined.
       */
    function (id) {
        var _this = this;
        console.log("init base-form.component");
        this.formType = global_constants_1.FormType.Insert;
        this.status = global_constants_1.PageStatus.Init;
        if (id && id > 0) {
            // si tenemos identificador, leemos el elemento del servidor
            this.dataService.get(id, this.paramsDataService)
                .subscribe(function (result) { return _this.postInit(result); }, function (error) { return _this.messages.set(global_constants_1.PageStatus.Error, error); });
        }
        else {
            this.postInit();
        }
    };
    /** Cuando se pulsa un botón del toolbar. */
    /** Cuando se pulsa un botón del toolbar. */
    BaseFormComponent.prototype.onToolbarButtonPressed = /** Cuando se pulsa un botón del toolbar. */
    function (button) {
        console.log("Button pressed:" + button);
        if (button === bit_toolbar_component_1.BitToolbarComponent.SAVE_BUTTON) {
            this.save();
        }
        else if (button === bit_toolbar_component_1.BitToolbarComponent.DELETE_BUTTON) {
            this.delete();
        }
        else {
            this.otherAction(button);
        }
    };
    /**
     * Inserta o actualiza el registro en el servidor.
     * Una vez insertado o actualizado, se invoca al método postSave con el resultado de la operación.
     */
    /**
       * Inserta o actualiza el registro en el servidor.
       * Una vez insertado o actualizado, se invoca al método postSave con el resultado de la operación.
       */
    BaseFormComponent.prototype.save = /**
       * Inserta o actualiza el registro en el servidor.
       * Una vez insertado o actualizado, se invoca al método postSave con el resultado de la operación.
       */
    function () {
        var _this = this;
        if (!this.myForm.valid) {
            var errores = custom_validators_service_1.CustomValidators.validateForm(this.myForm);
            this.messageService.sendError(new error_1.Error(null, 'Error al formulari', 'Error en el formulario', null, null, true, "", errores));
        }
        else {
            if (this.preSave()) {
                console.log('save', this.object);
                this.dataService.save(this.object, this.formType, this.paramsDataService)
                    .subscribe(function (result) { return _this.postSave(result); }, function (error) { _this.status = global_constants_1.PageStatus.Error; });
            }
        }
    };
    /** Elimina el registro en el servidor. Una vez eliminado, se invoca al método postDelete. */
    /** Elimina el registro en el servidor. Una vez eliminado, se invoca al método postDelete. */
    BaseFormComponent.prototype.delete = /** Elimina el registro en el servidor. Una vez eliminado, se invoca al método postDelete. */
    function () {
        var p = this.messageService.confirm("Està segur que vol eliminar aquest registre. Aquesta acciò no es podrà desfer?");
        var local = this;
        p.then(function (confirmacion) {
            if (confirmacion) {
                local.dataService.delete(local.object.id, local.paramsDataService)
                    .subscribe(function (result) {
                    local.postDelete();
                }, function (error) { local.status = global_constants_1.PageStatus.Error; });
            }
        });
    };
    /** Cualquier otra accion que no sea la típica de guardar o eliminar */
    /** Cualquier otra accion que no sea la típica de guardar o eliminar */
    BaseFormComponent.prototype.otherAction = /** Cualquier otra accion que no sea la típica de guardar o eliminar */
    function (action) {
        console.warn("otherAction " + action + " pressed");
    };
    /** Limpia el formulario, estableciendo los valores como si hubieramos entrado por primera vez. */
    /** Limpia el formulario, estableciendo los valores como si hubieramos entrado por primera vez. */
    BaseFormComponent.prototype.cleanForm = /** Limpia el formulario, estableciendo los valores como si hubieramos entrado por primera vez. */
    function () {
        //¿¿Algo que hacer??
    };
    Object.defineProperty(BaseFormComponent.prototype, "isEditMode", {
        /** Devuelve true si el formulario edita un elemento existente, o false si inserta un nuevo elemento. */
        get: /** Devuelve true si el formulario edita un elemento existente, o false si inserta un nuevo elemento. */
        function () {
            return (this.formType === global_constants_1.FormType.Update);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseFormComponent.prototype, "isDataChanged", {
        /**
         * Devuelve true si hay algún valor cambiado en la ficha.
         * Puede ser por un campo del formulario (this.myForm.dirty) o por modificaciones en elementos relacionados.
         */
        get: /**
           * Devuelve true si hay algún valor cambiado en la ficha.
           * Puede ser por un campo del formulario (this.myForm.dirty) o por modificaciones en elementos relacionados.
           */
        function () {
            return this.myForm.dirty;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseFormComponent.prototype, "isFormValid", {
        /** Devuelve true si el formulario es válido (campos validados automáticamente, elementos relacionados, etc.). */
        get: /** Devuelve true si el formulario es válido (campos validados automáticamente, elementos relacionados, etc.). */
        function () {
            //console.log('isFormValid', this.myForm.valid, this.myForm);
            return this.myForm.valid;
        },
        enumerable: true,
        configurable: true
    });
    /** Evalúa si el usuario puede desactivar/cerrar el componente sin perder cambios todavía no guardados. */
    /** Evalúa si el usuario puede desactivar/cerrar el componente sin perder cambios todavía no guardados. */
    BaseFormComponent.prototype.canDeactivate = /** Evalúa si el usuario puede desactivar/cerrar el componente sin perder cambios todavía no guardados. */
    function () {
        if (this.isDataChanged) {
            //si hay cambios, mostramos la ventana de diálogo
            var p = this.messageService.confirm("Hi ha canvis pendents de desar. Està segur de que vol sortir de la plana i perdre-los?");
            var o = Observable_1.Observable.fromPromise(p);
            return o;
        }
        else {
            // si no hay cambios, no la mostramos
            return true;
        }
    };
    /** Método que se ejecuta después de la lectura de los datos desde el servidor. */
    /** Método que se ejecuta después de la lectura de los datos desde el servidor. */
    BaseFormComponent.prototype.postInit = /** Método que se ejecuta después de la lectura de los datos desde el servidor. */
    function (result) {
        result = result || this.object;
        this.object = result;
        this.formType = (result && result.id) ? global_constants_1.FormType.Update : global_constants_1.FormType.Insert;
        if (this.toolbar[bit_toolbar_component_1.BitToolbarComponent.DELETE_BUTTON])
            this.toolbar[bit_toolbar_component_1.BitToolbarComponent.DELETE_BUTTON].visible = (this.formType == global_constants_1.FormType.Update && !this.dialogMode); // no permitimos borrar en modo diálogo
        if (this.toolbar[bit_toolbar_component_1.BitToolbarComponent.SAVE_BUTTON])
            this.toolbar[bit_toolbar_component_1.BitToolbarComponent.SAVE_BUTTON].enable = true; //this.isFormValid; //si el form es valido activamos el botón
        this.updateToolbar();
    };
    /** Método que se ejecuta antes de la inserción/modificación de los datos en el servidor. */
    /** Método que se ejecuta antes de la inserción/modificación de los datos en el servidor. */
    BaseFormComponent.prototype.preSave = /** Método que se ejecuta antes de la inserción/modificación de los datos en el servidor. */
    function () {
        // por ejemplo, limpiar objetos anidados que no tengan ID (workaround para formularios)
        return true;
    };
    /** Método que se ejecuta después de la inserción/modificación de los datos en el servidor. */
    /** Método que se ejecuta después de la inserción/modificación de los datos en el servidor. */
    BaseFormComponent.prototype.postSave = /** Método que se ejecuta después de la inserción/modificación de los datos en el servidor. */
    function (result, avoidMessage) {
        this.object = result;
        avoidMessage || this.messageService.sendNotification(new bitmessage_1.BitMessage("Informació", "Les dades s'han desat correctament."));
        if (this.formType == global_constants_1.FormType.Insert) {
            // navegamos a la ficha en modo edición. Esto permite que se sustituya la URL y evitamos efectos colaterales
            !this.dialogMode && this.router.navigate([general_utils_service_1.GeneralUtils.getPathFromURL(this.activatedRoute.snapshot.url), result.id], { preserveQueryParams: this.isPreserveQueryParams(), relativeTo: this.activatedRoute.parent });
            // si estamos en modo diálogo, hacemos un nuevo init con la entidad insertada
            this.dialogMode && this.init(result.id);
        }
        else {
            this.myForm.markAsPristine(); // el formulario vuelve a estar sincronizado con el servidor
            this.myForm.markAsUntouched(); // idem
        }
        this.onSave && this.onSave.emit(this.object);
    };
    /** Método que se ejecuta después de la eliminación del elemento en el servidor. */
    /** Método que se ejecuta después de la eliminación del elemento en el servidor. */
    BaseFormComponent.prototype.postDelete = /** Método que se ejecuta después de la eliminación del elemento en el servidor. */
    function () {
        this.messageService.sendNotification(new bitmessage_1.BitMessage("Informació", "Les dades s'han eliminat correctament."));
        !this.dialogMode && this.goBack();
    };
    Object.defineProperty(BaseFormComponent.prototype, "safeBreadcrumb", {
        /* Devuelve el breadcrumb solo si procede (por ejemplo, en modo diálogo no queremos breadcrumb). */
        get: /* Devuelve el breadcrumb solo si procede (por ejemplo, en modo diálogo no queremos breadcrumb). */
        function () {
            return !this.dialogMode && this.breadcrumb;
        },
        enumerable: true,
        configurable: true
    });
    // Métodos relacionados con pestañas (tabs) //
    /** Registra una pestaña de visualización de información. */
    // Métodos relacionados con pestañas (tabs) //
    /** Registra una pestaña de visualización de información. */
    BaseFormComponent.prototype.registerTab = 
    // Métodos relacionados con pestañas (tabs) //
    /** Registra una pestaña de visualización de información. */
    function (index, id) {
        this.myTabs[id] = index;
    };
    // inicialización de pestañas (si hay)
    // inicialización de pestañas (si hay)
    BaseFormComponent.prototype.initTabs = 
    // inicialización de pestañas (si hay)
    function () {
        this.initialTab = this.activatedRoute.snapshot.params["tab"];
        for (var tab in this.myTabs) {
            this.loadedTab[tab] = this.isInitialTab(tab);
        }
    };
    /** Indica si la pestaña correspondiente es la inicial. */
    /** Indica si la pestaña correspondiente es la inicial. */
    BaseFormComponent.prototype.isInitialTab = /** Indica si la pestaña correspondiente es la inicial. */
    function (tab) { return this.initialTab === tab; };
    /** Indica si la pestaña correspondiente está cargada. */
    /** Indica si la pestaña correspondiente está cargada. */
    BaseFormComponent.prototype.isLoadedTab = /** Indica si la pestaña correspondiente está cargada. */
    function (tab) { return this.loadedTab[tab]; };
    /** Controla cuando el usuario cambia de tab. */
    /** Controla cuando el usuario cambia de tab. */
    BaseFormComponent.prototype.changeTab = /** Controla cuando el usuario cambia de tab. */
    function (e) {
        console.log("change tab to number {}", e.index);
        this.currentTab = e.index;
        for (var tab in this.myTabs) {
            this.loadedTab[tab] = this.loadedTab[tab] || (e.index == this.myTabs[tab]);
        }
    };
    BaseFormComponent.prototype.isPreserveQueryParams = function () {
        return true;
    };
    BaseFormComponent.propDecorators = {
        "onSave": [{ type: core_1.Output },],
        "dialogMode": [{ type: core_1.Input },],
        "reset": [{ type: core_1.Input },],
    };
    return BaseFormComponent;
}(base_component_1.BaseComponent));
exports.BaseFormComponent = BaseFormComponent;
//# sourceMappingURL=base-form.component.js.map