"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var core_2 = require("@angular/core");
var forms_1 = require("@angular/forms");
var forms_2 = require("@angular/forms");
var configuration_editor_1 = require("../data/configuration-editor");
var noop = function () { };
var ɵ0 = noop;
exports.ɵ0 = ɵ0;
exports.CUSTOM_BITEDITOR_CONTROL_VALUE_ACCESSOR = {
    provide: forms_2.NG_VALUE_ACCESSOR,
    useExisting: core_1.forwardRef(function () { return BitEditorComponent; }),
    multi: true
};
/**
* Clase que utiliza un custom componente para campo editor de HTML
* Utiliza la interfaz ControlValueAccessor
*    <bit-editor [(ngModel)]="object.descripcion" [ngModelOptions]="{standalone: true}" height="250">&nbsp;&nbsp;Descripció</bit-editor>
* Para utilizar, importar tambien js y css.
*
* https://www.primefaces.org/primeng/#/editor
* https://quilljs.com/docs/modules/toolbar/
*
**/
var BitEditorComponent = (function () {
    function BitEditorComponent() {
        this.tipo = "default";
        // si es "custom", se utiliza el objeto de configuracion: configuration.
        // si es "default", no se utiliza ninguna configuracion (se muestran todos los controles)
        this.configuration = BitEditorComponent.globalConfiguration;
        this.placeholder = '';
        this.readOnly = false;
        this.height = 100;
        this.hideLabel = false;
        this.forceObligatorio = false;
        this.onChange = new core_2.EventEmitter();
        // bit-editor de un proyecto
        this.innerValue = '';
        this.onTouchedCallback = noop;
        this.onChangeCallback = noop;
    }
    Object.defineProperty(BitEditorComponent.prototype, "value", {
        //get accessor
        get: 
        //get accessor
        function () {
            return this.innerValue;
        },
        //set accessor including call the onchange callback
        set: 
        //set accessor including call the onchange callback
        function (v) {
            if (v !== this.innerValue) {
                this.innerValue = v;
            }
            this.onChangeCallback(v);
        },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(BitEditorComponent.prototype, "value_lectura", {
        get: function () {
            if (this.value == null) {
                return "";
            }
            else {
                return this.value;
            }
        },
        enumerable: true,
        configurable: true
    });
    BitEditorComponent.prototype.changed = function (event) {
        this.onTouchedCallback();
        this.onChangeCallback(this.innerValue);
        this.onChange.emit(event);
    };
    BitEditorComponent.prototype.onFocus = function () {
        this.onTouchedCallback();
        this.onChangeCallback(this.innerValue);
    };
    BitEditorComponent.prototype.onInit = function (quill) {
        /*var toolbar = quill.editor.getModule('toolbar');
                toolbar.addHandler('link', function(value) {
                    if (value) {
                        var href = prompt('Introduzca la URL');
                        this.quill.format('link', href);
                    } else {
                        this.quill.format('link', false);
                    }
                });*/
    };
    BitEditorComponent.prototype.writeValue = function (value) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    };
    BitEditorComponent.prototype.registerOnChange = function (fn) {
        this.onChangeCallback = fn;
    };
    BitEditorComponent.prototype.registerOnTouched = function (fn) {
        this.onTouchedCallback = fn;
    };
    Object.defineProperty(BitEditorComponent.prototype, "obligatorio", {
        // decidimos marcar el campo como obligatorio si se ha indicado a la fuerza (por parámetro)
        // o si el control de formulario asociado está saltando por ser obligatorio y no haberse rellenado
        get: 
        // decidimos marcar el campo como obligatorio si se ha indicado a la fuerza (por parámetro)
        // o si el control de formulario asociado está saltando por ser obligatorio y no haberse rellenado
        function () {
            return this.forceObligatorio || (this.control && this.control.errors && this.control.errors['required']);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BitEditorComponent.prototype, "configuracion", {
        get: function () {
            return this.configuration ? this.configuration : BitEditorComponent.globalConfiguration;
        },
        enumerable: true,
        configurable: true
    });
    BitEditorComponent.globalConfiguration = configuration_editor_1.ConfigurationEditor.newObject();
    BitEditorComponent.decorators = [
        { type: core_1.Component, args: [{
                    selector: 'bit-editor',
                    template: "\n            <label *ngIf=\"!hideLabel\"><ng-content></ng-content>&nbsp;<span *ngIf=\"obligatorio\" class=\"obligatorio fa fa-asterisk\"></span>&nbsp;&nbsp;<i *ngIf=\"ayuda\" class=\"btn-ayuda fa fa-question\" (click)=\"showAyuda();\"></i></label>\n            <p *ngIf=\"tipo.indexOf('lectura') == 0 || readOnly\" class=\"lectura\">\n                {{(value_lectura && value_lectura!='')?value_lectura:'&nbsp;'}}\n            </p>\n            <p-editor *ngIf=\"!readOnly && tipo=='custom'\" [(ngModel)]=\"value\"\n                styleClass=\"editor\" id=\"{{nombre}}\" name=\"{{nombre}}\"\n                    (onTextChange)=\"changed($event)\" (onInit)=\"onInit($event)\"\n                    [placeholder]=\"placeholder\" [style]=\"{'height':height+'px'}\">\n                <p-header>\n                    <span class=\"ql-formats\" *ngIf=\"configuracion.fontSize\">\n                        <select class=\"ql-size\">\n                            <option value=\"small\">Petita</option>\n                            <!-- Note a missing, thus falsy value, is used to reset to default -->\n                            <option selected>Mitjana</option>\n                            <option value=\"large\">Gran</option>\n                            <option value=\"huge\">Molt gran</option>\n                        </select>\n                    </span>\n                    <span class=\"ql-formats\" *ngIf=\"configuracion.fontFormat\">\n                        <button class=\"ql-bold\" aria-label=\"Bold\"></button>\n                        <button class=\"ql-italic\" aria-label=\"Italic\"></button>\n                        <button class=\"ql-underline\" aria-label=\"Underline\"></button>\n                        <button class=\"ql-strike\" aria-label=\"Strikethrough\"></button>\n                    </span>\n                    <span class=\"ql-formats\" *ngIf=\"configuracion.bullet\">\n                        <button class=\"ql-list\" value=\"ordered\" type=\"button\"></button>\n                        <button class=\"ql-list\" value=\"bullet\" type=\"button\"></button>\n                    </span>\n                    <span class=\"ql-formats\" *ngIf=\"configuracion.link\">\n                        <button class=\"ql-link\" aria-label=\"Insert Link\" type=\"button\"></button>\n                    </span>\n                </p-header>\n            </p-editor>\n            <p-editor *ngIf=\"!readOnly && (!tipo || tipo=='default')\" [(ngModel)]=\"value\"\n                styleClass=\"editor\" id=\"{{nombre}}\" name=\"{{nombre}}\" (onTextChange)=\"changed($event)\"\n                    [placeholder]=\"placeholder\" [style]=\"{'height':height+'px'}\">\n            </p-editor>\n            <control-messages *ngIf=\"control!=null\" [control]=\"control\" [field]=\"nombre\"></control-messages>\n",
                    providers: [exports.CUSTOM_BITEDITOR_CONTROL_VALUE_ACCESSOR]
                },] },
    ];
    /** @nocollapse */
    BitEditorComponent.ctorParameters = function () { return []; };
    BitEditorComponent.propDecorators = {
        "control": [{ type: core_2.Input },],
        "nombre": [{ type: core_2.Input },],
        "tipo": [{ type: core_2.Input },],
        "configuration": [{ type: core_2.Input },],
        "placeholder": [{ type: core_2.Input },],
        "readOnly": [{ type: core_2.Input },],
        "height": [{ type: core_2.Input },],
        "hideLabel": [{ type: core_2.Input },],
        "forceObligatorio": [{ type: core_2.Input },],
        "onChange": [{ type: core_2.Output },],
    };
    return BitEditorComponent;
}());
exports.BitEditorComponent = BitEditorComponent;
//# sourceMappingURL=bit-editor.component.js.map